import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
/**
 *  This is a convenience class for reading <code>int</code>s, <code>double</code>s and <code>String</code>s from the console.
 *  No exceptions are thrown.
 */
public class ConsoleInput {
  /**/
  private BufferedReader br;
  /**
   *  Open the input connection.
   */
  public ConsoleInput ( ) {
    /**/
    br=new BufferedReader( new InputStreamReader( System.in ) );
  }
  /**
   *  Close the input connection.
   */
  public void close () {
    /**/
    try { br.close(); }
    catch( IOException ioe ) { }
    /**/
    return;    
  }
  /**
   *  Read a <code>String</code>. The user will be prompted until a non-empty <code>String</code> is enterred.
   *
   *  @param  prompt The user prompt.
   *  @return A non-empty String.
   */
  public String readString ( String prompt ) {
    /**/
    String rv;
    /**/
    while ( true ) {
      System.out.print(prompt);
      rv=readLine();
      if ( rv != null ) break;
    }
    /**/
    return rv;
  }
  /**
   *  Read an <code>int</code>. The user will be prompted until a valid <code>int</code> is enterred.
   *
   *  @param  prompt The user prompt.
   *  @return A valid int.
   */
  public int readInt ( String prompt ) {
    /**/
    int rv;
    String s;
    /**/
    while ( true ) {
      s=readString(prompt);
      rv=0;
      try {
        rv=Integer.parseInt(s);
        break;
      }
      catch ( NumberFormatException nfe ) { }
    }
    /**/
    return rv;
  }
  /**
   *  Read a <code>double</code>. The user will be prompted until a valid <code>double</code> is enterred.
   *
   *  @param  prompt The user prompt.
   *  @return A valid double.
   */
  public double readDouble ( String prompt ) {
    /**/
    double rv;
    String s;
    /**/
    while ( true ) {
      s=readString(prompt);
      rv=0.0;
      try {
        rv=Double.parseDouble(s);
        break;
      }
      catch ( NumberFormatException nfe ) { }
    }
    /**/
    return rv;
  }
  /**/
  private String readLine () {
    /**/
    String rv;
    /**/
    rv=null;
    try { rv=br.readLine(); }
    catch( IOException ioe ) { }
    /**/
    if ( rv.equals("") ) return null;
    else                 return rv.trim();
  }
}
