import java.util.ArrayList;
/**/
public class Solitaire {
  /**/
  private ArrayList<String> helpFile;
  private ConsoleInput ci;
  private Deck deck;
  private PlayArea pa;
  /**/
  public static void main ( String [] arg ) {
    /**/
    PlayArea playA;
    Solitaire s;
    /**/
    System.out.println();
    System.out.println("S o l i t a i r e");
    System.out.println();
    System.out.println("Type \"h\" for help.");
    System.out.println();
    /**/
    s=new Solitaire();
    playA=s.getPlayArea();
    System.out.println(playA);
    /**/
    while ( true ) {
      s.doPlay();
      System.out.println(playA);
      if ( s.hasBeenWon() ) {
        System.out.println("Game is won!");
        System.out.println();
        return;
      }
    }
  }
  /**/
  public Solitaire () {
    /**/
    FileInput fi;
    /**/
    deck=new Deck();
    pa=new PlayArea(deck);
    ci=new ConsoleInput();
    /**/
    helpFile=new ArrayList<String> ();
    fi=new FileInput("help.txt");
    while ( !fi.eof() ) helpFile.add( fi.readLine() );
    fi.close();
  }
  /**/
  public PlayArea getPlayArea () {
    return pa;
  }
  /**/
  public void doPlay () {
    /**/
    int numTokens, len;
    char cmnd;
    String command;
    String [] tokens;
    /**/
    while ( true ) {
      command=ci.readString("? ");
      tokens=tokenize(command);
      numTokens=tokens.length;
      if ( numTokens == 0 ) continue;
      /**/
      cmnd=tokens[0].charAt(0);
      if      ( cmnd == 'h' ) {
        doHelp();
        continue;
      }
      else if ( cmnd == 'q' ) {
        ci.close();
        System.out.println();
        System.exit(0);
      }
      /**/
      else if ( cmnd == 'a' ) {
        doAbout();
        continue;
      }
      else if ( cmnd == 'd' ) {
        if ( !doDraw() ) continue;
        else             break;
      }
      else if ( cmnd == 'f' ) {
        if ( !doFlip() ) continue;
        else             break;
      }
      else if ( cmnd == 'p' ) {
        if ( !doMovePlay(tokens) ) continue;
        else                       break;
      }
      else if ( cmnd == 'r' ) {
        if ( !doMoveRow(tokens) ) continue;
        else                      break;
      }
      else if ( cmnd == 't' ) {
        if ( !doMoveDestination(tokens) ) continue;
        else                              break;
      }
      else {
         System.out.println("unrecognizable command");
         continue;
      }
    }
    /**/
    return;
  }
  /**/
  public boolean hasBeenWon () {
    /**/
    boolean rv;
    CardRows crs;
    DestinationDeck c, d, h, s;
    PlayDeck pd;
    TopDecks td;
    /**/
    td=pa.getTopDecks();
    /**/
    crs=pa.getCardRows();
    c=td.getClubs();
    d=td.getDiamonds();
    h=td.getHearts();
    s=td.getSpades();
    pd=td.getPlayDeck();
    /**/
    rv =   crs.haveNoDown()
       && ( pd.size() == 0 )
       && c.containsAce()
       && d.containsAce()
       && h.containsAce()
       && s.containsAce();
    /**/
    return rv;
  }
  /**/
  private void doAbout () {
    /**/
    String [] info= new String [] {
      "                                  " ,
      "            written by            " ,
      "        Stephen V. Harren         " ,
      "       http://www.harren.us       " ,
      "                                  " ,
      "This program is distributed as is." ,
      "     No liability is assumed.     " ,
      "                                  "
    };
    for ( int i=0; i<info.length; ++i ) System.out.println(info[i]);
    /**/
    return;
  }
  /**/
  private void doHelp() {
    /**/
    System.out.println();
    for ( int i=0; i<helpFile.size(); ++i ) System.out.println( helpFile.get(i) );
    System.out.println();
  }
  /**/
  private boolean doDraw () {
    /**/
    boolean rv;
    PlayDeck pd;
    /**/
    pd=pa.getTopDecks().getPlayDeck();
    if ( pd.canDraw() ) {
      pd.draw();
      rv=true;
      System.out.println();
    }
    else {
      System.out.println("cannot draw");
      rv=false;
    }
    /**/
    return rv;
  }
  /**/
  private boolean doFlip () {
    /**/
    boolean rv;
    PlayDeck pd;
    /**/
    pd=pa.getTopDecks().getPlayDeck();
    if ( pd.canFlip() ) {
      pd.flip();
      rv=true;
      System.out.println();
    }
    else {
      System.out.println("cannot flip");
      rv=false;
    }
    /**/
    return rv;
  }
  /**/
  private boolean doMovePlay ( String [] arg ) {
    /**/
    int suitToMove, row;
    char fc;
    Card toMove;
    CardRow cr;
    CardRows crs;
    DestinationDeck dd;
    PlayDeck pd;
    String rowS;
    TopDecks td;
    CardRow [] cra;
    /**/
    if ( arg.length != 2 ) {
      System.out.println("one argument expected");
      return false;
    }
    /**/
    td=pa.getTopDecks();
    pd=td.getPlayDeck();
    if ( !pd.canRemove() ) {
      System.out.println("up play deck is empty");
      return false;
    }
    toMove=pd.getTopCard();
    suitToMove=toMove.getSuit();
    /**/
    fc=arg[1].charAt(0);
    if      ( fc == 'd' ) {
      /**/
      if      ( suitToMove == 0 ) dd=td.getClubs();
      else if ( suitToMove == 1 ) dd=td.getDiamonds();
      else if ( suitToMove == 2 ) dd=td.getHearts();
      else                        dd=td.getSpades();
      /**/
      if ( !dd.canAdd(toMove) ) {
        System.out.println( "cannot move card \"" + toMove.toString() + "\" to destination deck" );
        return false;
      }
      else {
        dd.add(pd.remove());
        System.out.println();
        return true;
      }
    }
    else if ( fc == 'r' ) {
      /**/
      if ( arg[1].length() != 2 ) {
        System.out.println("invalid argument");
        return false;
      }
      rowS=arg[1].substring(1,2);
      row=-1;
      try {
        row=Integer.parseInt(rowS);
      }
      catch ( NumberFormatException nfe ) {
        System.out.println("invalid argument");
        return false;
      }
      /**/
      if ( ( row < 0 ) || ( row > 6 ) ) {
        System.out.println("invalid row #");
        return false;
      }
      /**/
      crs=pa.getCardRows();
      cra=crs.getCardRows();
      cr=cra[row];
      /**/
      if ( !cr.canAdd(toMove) ) {
        System.out.println( "cannot add card \"" + toMove.toString() + "\" to row deck " + row );
        return false;
      }
      else {
        cr.add(pd.remove());
        System.out.println();
        return true;
      }
    }
    else {
      System.out.println("invalid argument");
      return false;
    }
  }
  /**/
  private boolean doMoveDestination ( String [] arg ) {
    /**/
    int row;
    char dest;
    Card toMove;
    CardRow cr;
    CardRow [] cra;
    CardRows crs;
    DestinationDeck dd;
    String rowS;
    TopDecks td;
    /**/
    if ( arg.length != 2 ) {
      System.out.println("one argument expected");
      return false;
    }
    if ( arg[0].length() != 2 ) {
      System.out.println("unrecognizable command");
      return false;
    }
    /**/
    td=pa.getTopDecks();
    dest=arg[0].charAt(1);
    /**/
    if      ( dest == 'c' ) dd=td.getClubs();
    else if ( dest == 'd' ) dd=td.getDiamonds();
    else if ( dest == 'h' ) dd=td.getHearts();
    else if ( dest == 's' ) dd=td.getSpades();
    else {
      System.out.println("invalid destination deck");
      return false;
    }
    /**/
    if ( !dd.canRemove() ) {
      System.out.println("cannot remove card from an empty destination deck");
      return false;
    }
    toMove=dd.getLastCard();
    /**/
    if ( arg[1].charAt(0) != 'r' ) {
      System.out.println("invalid argument");
      return false;
    }
    if ( arg[1].length() != 2 ) {
      System.out.println("invalid argument");
      return false;
    }
    /**/
    rowS=arg[1].substring(1,2);
    row=-1;
    try {
      row=Integer.parseInt(rowS);
    }
    catch ( NumberFormatException nfe ) {
      System.out.println("invalid argument");
      return false;
    }
    /**/
    if ( ( row < 0 ) || ( row > 6 ) ) {
      System.out.println("invalid row #");
      return false;
    }
    /**/
    crs=pa.getCardRows();
    cra=crs.getCardRows();
    cr=cra[row];
    /**/
    if ( !cr.canAdd(toMove) ) {
      System.out.println( "cannot add card \"" + toMove.toString() + "\" to row deck " + row );
      return false;
    }
    /**/
    dd.remove();
    cr.add(toMove);
    System.out.println();
    return true;
  }
  /**/
  private boolean doMoveRow ( String [] arg ) {
    /**/
    int row, fromValue, fromSize, fromSuit, toRow;
    char argOption;
    ArrayList<Card> alc;
    Card fromCard;
    CardRow crFrom, cr;
    CardRow [] cra;
    CardRows crs;
    DestinationDeck dd;
    String rowS, toRowS;
    TopDecks td;
    /**/
    if ( arg.length != 3 ) {
      System.out.println("two arguments expected");
      return false;
    }
    /**/
    if ( arg[0].length() != 2 ) {
      System.out.println("unrecognizable command");
      return false;
    }
    /**/
    rowS=arg[0].substring(1,2);
    row=-1;
    try {
      row=Integer.parseInt(rowS);
    }
    catch ( NumberFormatException nfe ) {
      System.out.println("unrecognizable command");
      return false;
    }
    /**/
    if ( ( row < 0 ) || ( row > 6 ) ) {
      System.out.println("invalid row #");
      return false;
    }
    /**/
    crFrom=(pa.getCardRows().getCardRows())[row];
    if ( crFrom.getBottomCard() == null ) {
      System.out.println( "card row " + row + " is empty" );
      return false;
    }
    /**/
    if ( !Card.isValidValueString(arg[1]) ) {
      System.out.println("invalid argument");
      return false;
    }
    /**/
    fromValue=Card.getValueFromString(arg[1]);
    fromCard=crFrom.getCardFromValue(fromValue);
    /**/
    if ( fromCard == null ) {
      System.out.println( "card not found in row deck " + row );
      return false;
    }
    /**/
    td=pa.getTopDecks();
    fromSize=crFrom.getReturnListSize(fromCard);
    argOption=arg[2].charAt(0);
    if      ( argOption == 'd' ) {
      /**/
      if ( fromSize != 1 ) {
        System.out.println("cannot add multiple cards to destination deck");
        return false;
      }
      /**/
      fromSuit=fromCard.getSuit();
      if      ( fromSuit == 0 ) dd=td.getClubs();
      else if ( fromSuit == 1 ) dd=td.getDiamonds();
      else if ( fromSuit == 2 ) dd=td.getHearts();
      else                      dd=td.getSpades();
      /**/
      if ( !dd.canAdd(fromCard) ) {
        System.out.println( "cannot add \"" + fromCard.toString() + "\" to destination deck" );
        return false;
      }
      /**/
      alc=crFrom.remove(fromCard);
      dd.add(alc.get(0));
      System.out.println();
      return true;
    }
    else if ( argOption == 'r' ) {
      /**/
      if ( arg[2].length() != 2 ) {
        System.out.println("invalid argument");
        return false;
      }
      /**/
      toRowS=arg[2].substring(1,2);
      toRow=-1;
      try {
        toRow=Integer.parseInt(toRowS);
      }
      catch ( NumberFormatException nfe ) {
        System.out.println("malformed argument");
        return false;
      }
      /**/
      if ( ( toRow < 0 ) || ( toRow > 6 ) ) {
        System.out.println("invalid row #");
        return false;
      }
      /**/
      crs=pa.getCardRows();
      cra=crs.getCardRows();
      cr=cra[toRow];
      if ( !cr.canAdd(fromCard) ) {
        System.out.println( "cannot add card \"" + fromCard.toString() + "\" to row deck " + toRow );
        return false;
      }
      /**/
      alc=crFrom.remove(fromCard);
      cr.add(alc);
      System.out.println();
      return true;
    }
    else {
      System.out.println("bad argument");
      return false;
    }
  }
  /**/
  private static String [] tokenize ( String s ) {
    /**/
    int numTokens, nzt, k;
    String [] tokens, rv;
    /**/
    tokens=s.split(" ");
    numTokens=tokens.length;
    /**/
    nzt=0;
    for ( int i=0; i<numTokens; ++i ) {
      if ( !tokens[i].equals("") ) ++nzt;
    }
    rv=new String [nzt];
    k=0;
    for ( int i=0; i<numTokens; ++i ) {
      if ( !tokens[i].equals("") ) rv[k++]=tokens[i];
    }
    /**/
    return rv;
  }
}
